module Main where

import Control.Monad.IO.Class
import Data.Array.ESFA.Spec
import Data.Array.ESFA.SimArch

main :: IO ()
main = runtree 5 () script

script :: StESFA () Int ()
script =
  do liftIO $ putStrLn "\nESFA program Example1"

-- You first have to initialize the system
     esfmInitialize

-- Create array a1 by updating empty with value 101 at index 100.
-- The result is Just a1, and a1 is an AName; if failure, result is Nothing
-- The arguments are
--   "a1" is a label that helps provide more readable output (see below)
--   esfmEmpty is the array that is being updated
--   100 is the index
--   101 is the element value
-- The result is something like a1 := empty; a1[100] := 101

     Just a1 <- esfmUpdate "a1" esfmEmpty (100, mkInt 101)

-- Now define a2 to be the same as a1, except a2[200] = 201
     Just a2 <- esfmUpdate "a2" a1 (200, mkInt 201)

-- Print a1[100] which should be 101
     Just x <- esfmLookup "a1" a1 100
     liftIO (putStrLn ("a1[100] = " ++ show x))

-- Print a2[100] (expect 100) and a2[200] (expect 201)
     Just y1 <- esfmLookup "a2" a2 100    -- y1 = a2[100]
     liftIO (putStrLn ("y1 = " ++ show y1))
     Just y2 <- esfmLookup "a2" a2 200    -- y2 = a2[200]
     liftIO (putStrLn ("y2 = " ++ show y2))

     liftIO (putStrLn "\nDoing esfmPrintSpecArrays")
     esfmPrintSpecArrays

     liftIO (putStrLn "\nDoing esfmPrintMachineArrays")
     esfmPrintMachineArrays

     liftIO (putStrLn "\nDoing esfmPrintState")
     esfmPrintState

     return ()


-- Notes: here we're is using esfmUpdate, which is not the primitive
-- update operation; it also keeps track of some additional
-- information useful for printing diagnostics.  The string parameter
-- "a1" is just used to make the output more readable.  The operation
-- could fail, in which case it returns Nothing and the Just a1
-- pattern would then result in a failure of the program.  A robust
-- program would, of course, handle errors better.


mkInt :: Int -> EsfEltVal Int
mkInt n = EsfUserOrdVal n
getInt :: EsfEltVal Int -> Int
getInt (EsfUserOrdVal n) = n
getInt _ = error "element should be EsfUserOrdVal but isn't"
